@@ -258,6 +258,24 @@ class AppData(aem.Codecs):
 	def referencebyname(self):
 		self.connect()
 		return self._referencebyname
+		
+	_kYes = Keyword('yes')
+	_kNo = Keyword('no')
+	_kAsk = Keyword('ask')
+	
+	def permissiontoautomate(self, code=kae.typeWildCard*2):
+		try:
+			self.target().permissiontoautomate(code[:4], code[4:], True)
+		except MacOSError, e:
+			if e.args[0] == -1744: # errAEEventWouldRequireUserConsent
+				return self._kAsk # unknown (i.e. user has yet to allow/forbid access)
+			elif e.args[0] == -1743: # errAEEventNotPermitted (broken in 10.14.x; see <rdar://44049802>)
+				return self._kNo
+			else: # e.g. -600 (process not found)
+				raise
+		except NotImplementedError: # always True if OS version < 10.14
+			pass
+		return self._kYes
 	
 	##
 	
@@ -405,6 +423,10 @@ class Command(_Base):
 			timeout *= 60 # convert to ticks
 		# ignore application's reply?
 		sendflags = keywordargs.pop('waitreply', True) and kae.kAEWaitReply or kae.kAENoReply
+		# allow macOS 10.14+ to prompt user for access to target app, if needed, before sending it this event (if False, throws CommandError -1744 if user hasn't yet granted permission)
+		canaskforconsent = bool(keywordargs.pop('canaskforconsent', True))
+		if not canaskforconsent:
+			sendflags |= kAEDoNotPromptForUserConsent
 		atts, params = {'subj':None}, {}
 		# add considering/ignoring attributes (note: most apps currently ignore these)
 		ignoreoptions = keywordargs.pop('ignore', None)
@@ -499,6 +521,8 @@ class Command(_Base):
 	
 	def AS_formatcommand(self, args):
 		return '%r(%s)' % (self, ', '.join(['%r' % (v,) for v in args[0]] + ['%s=%r' % (k, v) for (k, v) in args[1].items()]))
+	
+	permissiontoautomate = property(lambda self: self.AS_appdata.permissiontoautomate(self._code))
 		
 
 ######################################################################
@@ -576,6 +600,10 @@ class Reference(_Base):
 		try:
 			selectortype, code = self.AS_appdata.referencebyname()[name]
 		except KeyError:
+			# properties and __getattr__ do not mix in Python 2.x (if property's getter throws an error, __getattribute__ calls __getattr__ instead), so Application.permissiontoautomate property must be kludged here
+			if name == 'permissiontoautomate' and isinstance(self, Application):
+				return self.AS_appdata.permissiontoautomate()
+
 			raise AttributeError("Unknown property, element or command: %r" % name)
 		if selectortype == kProperty:
 			return Reference(self.AS_appdata, self.AS_aemreference.property(code))
@@ -827,7 +855,7 @@ class CommandError(Exception):
 		if isinstance(self.realerror, aem.EventError):
 			return int(self.realerror)
 		elif isinstance(self.realerror, MacOSError):
-			return self.realerror[0]
+			return self.realerror.args[0]
 		else:
 			return -2700
 	
