@@ -4,7 +4,6 @@
  *
  * Docstring descriptions taken from Apple developer documentation
  *     Copyright (C) 1993-2008 Apple Inc.
- *
  */
  
 /* =========================== Module AE =========================== */
@@ -13,7 +12,6 @@
 
 #define AE_MODULE_C
 #include "ae.h"
-#include "sendthreadsafe.c"
 
 // Event handling
 #if __LP64__
@@ -26,6 +24,12 @@ AEEventHandlerUPP upp_GenericEventHandler;
 AECoercionHandlerUPP upp_GenericCoercionHandler;
 
 
+extern OSStatus AEDeterminePermissionToAutomateTarget(const AEAddressDesc* target, 
+													  AEEventClass theAEEventClass, 
+													  AEEventID theAEEventID, 
+													  Boolean askUserIfNeeded) __attribute__((weak_import));
+
+
 /* --------------------------- MacOSError ---------------------------- */
 
 /* Exception object used by AE extension for OS errors */
@@ -362,10 +366,10 @@ static PyObject *AEDesc_AESendMessage(AEDescObject *_self, PyObject *_args) // t
 						  &timeOutInTicks))
 		return NULL;
 	Py_BEGIN_ALLOW_THREADS
-	_err = AE_SendMessageThreadSafe(&_self->ob_itself,
-									&reply,
-									sendMode,
-									timeOutInTicks);
+	_err = AESendMessage(&_self->ob_itself,
+						 &reply,
+						 sendMode,
+						 timeOutInTicks);
 	Py_END_ALLOW_THREADS
 	if (_err != noErr) return AE_MacOSError(_err);
 	_res = Py_BuildValue("O&",
@@ -373,6 +377,35 @@ static PyObject *AEDesc_AESendMessage(AEDescObject *_self, PyObject *_args) // t
 	return _res;
 }
 
+static PyObject *AEDesc_AEDeterminePermissionToAutomateTarget(AEDescObject *_self, PyObject *_args) // thread-safe
+{
+	PyObject *_res = NULL;
+	AEEventClass theAEEventClass;
+	AEEventID theAEEventID;
+	Boolean askUserIfNeeded;
+	OSStatus _err;
+
+	if (AEDeterminePermissionToAutomateTarget == NULL) {
+		PyErr_SetString(PyExc_NotImplementedError,
+						"aem.ae.AEDesc.permissiontoautomate isn't available in 10.13 or earlier.");
+		return NULL;
+	}
+	if (!PyArg_ParseTuple(_args, "O&O&b",
+	                      AE_GetOSType, &theAEEventClass,
+	                      AE_GetOSType, &theAEEventID,
+	                      &askUserIfNeeded))
+		return NULL;
+	_err = AEDeterminePermissionToAutomateTarget(&_self->ob_itself, 
+												 theAEEventClass, 
+												 theAEEventID, 
+												 askUserIfNeeded);
+	if (_err != noErr) return AE_MacOSError(_err);
+	Py_INCREF(Py_None);
+	_res = Py_None;
+	return _res;
+}
+
+
 static PyMethodDef AEDesc_methods[] = {
 	
 	{"flatten", (PyCFunction)AEDesc_AEFlattenDesc, 1, PyDoc_STR(
@@ -438,6 +471,20 @@ static PyMethodDef AEDesc_methods[] = {
 		"-> (AppleEvent reply)\n"
 		"Sends an AppleEvent to a target process.")},
 	
+	{"permissiontoautomate", (PyCFunction)AEDesc_AEDeterminePermissionToAutomateTarget, 1, PyDoc_STR(
+		"D.permissiontoautomate(AEEventClass theAEEventClass,\n"
+		"                       AEEventID theAEEventID,\n"
+		"                       Boolean askUserIfNeeded)\n"
+		"Throws if current process is not authorized to send an Apple event\n"
+		"to the target application. Descriptor must be an AEAddressDesc.\n"
+		"Errors include:\n"
+		"errAEEventNotPermitted (-1743): the user has declined permission.\n"
+    	"errAEEventWouldRequireUserConsent (-1744): user consent is required\n"
+    	"  for this, but the user has not yet been prompted for it. You need\n"
+    	"  to pass False for askUserIfNeeded to get this.\n"
+    	"procNotFound (-600): the specified app is not currently running.\n"
+    	"Throws NotImplementedError on 10.13 and earlier.")},
+	
 	{NULL, NULL, 0}
 };
 
@@ -1112,6 +1159,7 @@ static PyObject *AE_LaunchApplication(PyObject* self, PyObject* args)
 		return NULL;
 	LSApplicationParameters appParams = {0, flags, &appRef, NULL, NULL, NULL, &firstEvent};
 	err = LSOpenApplication(&appParams, &psn);
+	if (err != noErr) return AE_MacOSError(err);
 	err = AECreateDesc(typeProcessSerialNumber,
 	                   &psn, sizeof(psn),
 	                   &result);
