@@ -258,6 +258,24 @@ class AppData(aem.Codecs):
 	def referencebyname(self):
 		self.connect()
 		return self._referencebyname
+		
+	_kYes = Keyword('yes')
+	_kNo = Keyword('no')
+	_kAsk = Keyword('ask')
+	
+	def permissiontoautomate(self, code=kae.typeWildCard*2):
+		try:
+			self.target().permissiontoautomate(code[:4], code[4:], True)
+		except MacOSError as e:
+			if e.args[0] == -1744: # errAEEventWouldRequireUserConsent
+				return self._kAsk # unknown (i.e. user has yet to allow/forbid access)
+			elif e.args[0] == -1743: # errAEEventNotPermitted (broken in 10.14.x; see <rdar://44049802>)
+				return self._kNo
+			else: # e.g. -600 (process not found)
+				raise
+		except NotImplementedError: # always True if OS version < 10.14
+			pass
+		return self._kYes
 	
 	##
 	
@@ -403,6 +421,10 @@ class Command(_Base):
 			timeout *= 60 # convert to ticks
 		# ignore application's reply?
 		sendflags = keywordargs.pop('waitreply', True) and kae.kAEWaitReply or kae.kAENoReply
+		# allow macOS 10.14+ to prompt user for access to target app, if needed, before sending it this event (if False, throws CommandError -1744 if user hasn't yet granted permission)
+		canaskforconsent = bool(keywordargs.pop('canaskforconsent', True))
+		if not canaskforconsent:
+			sendflags |= kAEDoNotPromptForUserConsent
 		atts, params = {b'subj':None}, {}
 		# add considering/ignoring attributes (note: most apps currently ignore these)
 		ignoreoptions = keywordargs.pop('ignore', None)
@@ -497,6 +519,8 @@ class Command(_Base):
 	
 	def AS_formatcommand(self, args):
 		return '{!r}({})'.format(self, ', '.join(['{!r}'.format(v) for v in args[0]] + ['{}={!r}'.format(k, v) for (k, v) in args[1].items()]))
+	
+	permissiontoautomate = property(lambda self: self.AS_appdata.permissiontoautomate(self._code))
 		
 
 ######################################################################
@@ -785,7 +809,9 @@ class Application(Reference):
 			aem.Application.launch(self.AS_appdata.identifier)
 			self.AS_appdata.target().reconnect() # make sure aem.Application object's AEAddressDesc is up to date
 		else: # send launch event to app (will error if not already running)
-			Command(self, 'launch', 'ascrnoop', {})()
+			Command(self, 'launch', b'ascrnoop', {})()
+	
+	permissiontoautomate = property(lambda self: self.AS_appdata.permissiontoautomate())
 
 
 #######
@@ -825,7 +851,7 @@ class CommandError(Exception):
 		if isinstance(self.realerror, aem.EventError):
 			return int(self.realerror)
 		elif isinstance(self.realerror, MacOSError):
-			return self.realerror[0]
+			return self.realerror.args[0]
 		else:
 			return -2700
 	
